/**
 * @file
 * Installs @sentry/browser and updates library version.
 */

const Bourne = require('@hapi/bourne');
const crypto = require('crypto');
const fs = require('fs');
const got = require('got');
const stream = require('stream');
const pipeline = require('util').promisify(stream.pipeline);
const process = require('process');
const yaml = require('js-yaml');
let version;
const readJson = fs.promises.readFile('package-lock.json')
  .then((contents) => version = JSON.parse(contents).dependencies['@sentry/tracing'].version);
const librariesFile = 'raven.libraries.yml';
let libraries;
const readYaml = fs.promises.readFile(librariesFile)
  .then((contents) => libraries = yaml.load(contents));
Promise.all([readJson, readYaml])
  .then(() => {
    libraries['sentry-browser'].version = version;
    const writeVersion = fs.promises.writeFile(librariesFile, yaml.dump(libraries));
    const destination = 'js/';
    const jsFile = 'bundle.tracing.min.js';
    const url = ['https://browser.sentry-cdn.com', version, jsFile].join('/');
    const jsHash = crypto.createHash('sha384');
    const mapHash = crypto.createHash('sha384');
    const fetchJs = pipeline(
      got.stream(url),
      new stream.Transform({
        transform(data, encoding, callback) {
          jsHash.update(data);
          this.push(data);
          callback();
        }
      }),
      fs.createWriteStream(destination + jsFile)
    );
    const mapSuffix = '.map';
    const fetchMap = pipeline(
      got.stream(url + mapSuffix),
      new stream.Transform({
        transform(data, encoding, callback) {
          mapHash.update(data);
          this.push(data);
          callback();
        }
      }),
      fs.createWriteStream(destination + jsFile + mapSuffix)
    );
    const releaseUrl = ['https://release-registry.services.sentry.io/sdks/sentry.javascript.browser', version].join('/');
    let releaseData;
    const fetchReleaseData = got(releaseUrl, {
      parseJson: text => Bourne.parse(text)
    }).json()
      .then((parsed) => releaseData = parsed);
    Promise.all([fetchJs, fetchMap, writeVersion, fetchReleaseData])
      .then(() => {
        const calculatedJsHash = jsHash.digest('base64');
        const calculatedMapHash = mapHash.digest('base64');
        const publishedJsHash = releaseData.files[jsFile].checksums['sha384-base64'];
        const publishedMapHash = releaseData.files[jsFile + mapSuffix].checksums['sha384-base64'];
        console.log('Verifying ' + jsFile + '...');
        console.log('calculated: ' + calculatedJsHash);
        console.log(' published: ' + publishedJsHash);
        if (calculatedJsHash !== publishedJsHash) {
          throw new Error('Hash mismatch!');
        }
        console.log('Verifying ' + jsFile + mapSuffix + '...');
        console.log('calculated: ' + calculatedMapHash);
        console.log(' published: ' + publishedMapHash);
        if (calculatedMapHash !== publishedMapHash) {
          throw new Error('Hash mismatch!');
        }
        console.log('Achievement unlocked.');
      })
      .catch(error => {
        console.error(error.message);
        process.exit(1);
      });
  });
